from sparkdesk.chat_utils import limit_chat_history

from ..chat import BaseChat
from .completion import ChatCompletion


class Chat(BaseChat):
    """
    Class for creating chat
    """

    completion_cls = ChatCompletion
    chat_completion: ChatCompletion

    async def ask(self, question: str):
        """
        Appends the user's question to the chat context,
        generates responses using the chat_completion model,
        and adds the assistant's response to the chat context.

        Args:
            question (str): The user's question.

        Yields:
            dict: A dictionary representing each response generated by
                the chat_completion model.
            reference: https://www.xfyun.cn/doc/spark/Web.html
        """

        self.chat_history.append({"role": "user", "content": question})
        self.chat_history = limit_chat_history(self.chat_history)

        response_messages = []
        responses = self.chat_completion.create(self.chat_history)
        try:
            async for response in responses:
                response_messages.extend(
                    [e["content"] for e in response["payload"]["choices"]["text"]]
                )
                yield response
        finally:  # GeneratorExit
            self.chat_history.append(
                {"role": "assistant", "content": "".join(response_messages)}
            )



import logging
from typing import Any, AsyncIterator, Dict, List

from websockets import connect

from sparkdesk.completion import BaseChatCompletion

logger = logging.getLogger(__name__)


class ChatCompletion(BaseChatCompletion):
    async def create(
        self, message_text: List[Dict[str, str]]
    ) -> AsyncIterator[Dict[str, Any]]:
        """
        Create chat completion.
        reference: https://www.xfyun.cn/doc/spark/Web.html

        Args:
            message_text:
                A list of dictionaries containing the message content.
                The total length of the message content should be less than 8192.
                one token is approximately equal to 1.5 Chinese characters or
                0.8 English words.
                ex: [{"role": "user", "content": "Hi"}]

        Yields:
            An iterator that yields the processed response messages from the websocket.
        """
        async with connect(self.authentication.get_authorization_url()) as websocket:
            input_message = self._build_ws_input_message(message_text)
            logger.debug(f"send: {input_message}")
            await websocket.send(input_message)

            response_message = await websocket.recv()
            logger.debug(f"recv: {response_message}")
            while True:
                processed_ws_response_message = self._process_ws_response_message(
                    response_message
                )
                yield processed_ws_response_message
                # if last message, exit
                if processed_ws_response_message["header"]["status"] == 2:
                    break
                response_message = await websocket.recv()
                logger.debug(f"recv: {response_message}")
